__all__ = ["Client", "InvalidKey", "NothingFound", "UnexpectedResponse", "YandexGeocoderException"]

from yandex_geocoder.client import Client
from yandex_geocoder.exceptions import (
    InvalidKey,
    NothingFound,
    UnexpectedResponse,
    YandexGeocoderException,
)



__all__ = ["YandexGeocoderException", "UnexpectedResponse", "NothingFound", "InvalidKey"]


class YandexGeocoderException(Exception):
    pass


class UnexpectedResponse(YandexGeocoderException):
    pass


class NothingFound(YandexGeocoderException):
    pass


class InvalidKey(YandexGeocoderException):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yandex_geocoder']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'yandex-geocoder',
    'version': '3.0.1',
    'description': 'Simple library for getting address or coordinates via Yandex geocoder',
    'long_description': '# Yandex Geocoder\n\nGet address coordinates via Yandex geocoder\n\n[![test](https://github.com/sivakov512/yandex-geocoder/workflows/test/badge.svg)](https://github.com/sivakov512/yandex-geocoder/actions?query=workflow%3Atest)\n[![Coverage Status](https://coveralls.io/repos/github/sivakov512/yandex-geocoder/badge.svg?branch=master)](https://coveralls.io/github/sivakov512/yandex-geocoder?branch=master)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n[![Python versions](https://img.shields.io/pypi/pyversions/yandex-geocoder.svg)](https://pypi.python.org/pypi/yandex-geocoder)\n[![PyPi](https://img.shields.io/pypi/v/yandex-geocoder.svg)](https://pypi.python.org/pypi/yandex-geocoder)\n\n## Installation\n\nInstall it via `pip` tool:\n\n```shell\npip install yandex-geocoder\n```\n\nor Poetry:\n\n```shell\npoetry add yandex-geocoder\n```\n\n## Usage example\n\nYandex Geocoder requires an API developer key, you can get it [here](https://developer.tech.yandex.ru/services/) to use this library.\n\n```python\nfrom decimal import Decimal\n\nfrom yandex_geocoder import Client\n\n\nclient = Client("your-api-key")\n\ncoordinates = client.coordinates("Москва Льва Толстого 16")\nassert coordinates == (Decimal("37.587093"), Decimal("55.733969"))\n\naddress = client.address(Decimal("37.587093"), Decimal("55.733969"))\nassert address == "Россия, Москва, улица Льва Толстого, 16"\n```\n\n## Development and contribution\n\nFirst of all you should install [Poetry](https://python-poetry.org).\n\n- install project dependencies\n\n```bash\nmake install\n```\n\n- run linters\n\n```bash\nmake lint\n```\n\n- run tests\n\n```bash\nmake test\n```\n\n- feel free to contribute!\n',
    'author': 'Nikita Sivakov',
    'author_email': 'sivakov512@icloud.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/sivakov512/yandex-geocoder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



__all__ = ["Client"]

import dataclasses
import typing
from decimal import Decimal

import requests

from .exceptions import InvalidKey, NothingFound, UnexpectedResponse


@dataclasses.dataclass
class Client:
    """Yandex geocoder API client.

    :Example:
        >>> from yandex_geocoder import Client
        >>> client = Client("your-api-key")

        >>> coordinates = client.coordinates("Москва Льва Толстого 16")
        >>> assert coordinates == (Decimal("37.587093"), Decimal("55.733969"))

        >>> address = client.address(Decimal("37.587093"), Decimal("55.733969"))
        >>> assert address == "Россия, Москва, улица Льва Толстого, 16"

    """

    __slots__ = ("api_key",)

    api_key: str

    def _request(self, address: str) -> dict[str, typing.Any]:
        response = requests.get(
            "https://geocode-maps.yandex.ru/1.x/",
            params=dict(format="json", apikey=self.api_key, geocode=address),
        )

        if response.status_code == 200:
            got: dict[str, typing.Any] = response.json()["response"]
            return got
        elif response.status_code == 403:
            raise InvalidKey()
        else:
            raise UnexpectedResponse(
                f"status_code={response.status_code}, body={response.content!r}"
            )

    def coordinates(self, address: str) -> tuple[Decimal, ...]:
        """Fetch coordinates (longitude, latitude) for passed address."""
        data = self._request(address)["GeoObjectCollection"]["featureMember"]

        if not data:
            raise NothingFound(f'Nothing found for "{address}" not found')

        coordinates: str = data[0]["GeoObject"]["Point"]["pos"]
        longitude, latitude = tuple(coordinates.split(" "))

        return Decimal(longitude), Decimal(latitude)

    def address(self, longitude: Decimal, latitude: Decimal) -> str:
        """Fetch address for passed coordinates."""
        data = self._request(f"{longitude},{latitude}")["GeoObjectCollection"]["featureMember"]

        if not data:
            raise NothingFound(f'Nothing found for "{longitude} {latitude}"')

        got: str = data[0]["GeoObject"]["metaDataProperty"]["GeocoderMetaData"]["text"]
        return got
